@@ -158,7 +158,7 @@
 #'  If the age distribution of assigned parents does not capture the maximum
 #'  possible age of parents, it is advised to specify \code{MaxAgeParent} for
 #'  one or both sexes. Not doing so may hinder subsequent assignment of both
-#'  dummy parents and grandparents.
+#'  dummy parents and grandparents. Not compatible with \code{Smooth}.
 #'
 #'  @section grandparents & avuncular
 #'  The agepriors for grand-parental and avuncular pairs is calculated from
@@ -172,7 +172,6 @@
 #'
 #' @examples
 #' # without pedigree or lifehistdata:
-#' MakeAgePrior()
 #' MakeAgePrior(MaxAgeParent = c(2,3))
 #' MakeAgePrior(Discrete=TRUE)
 #'
@@ -181,7 +180,6 @@
 #'   BirthYear=1984))
 #'
 #' # overlapping generations:
-#' data(Ped_griffin, SeqOUT_griffin, package="sequoia")
 #' # without pedigree: MaxAgeParent = max age difference between any pair +1
 #' MakeAgePrior(LifeHistData = SeqOUT_griffin$LifeHist)
 #' # with pedigree:
@@ -193,6 +191,11 @@
 #'              LifeHistData=SeqOUT_griffin$LifeHist,
 #'              Smooth=TRUE, Flatten=TRUE)
 #'
+#' # Call from sequoia() via args.AP:
+#' Seq_HSg5 <- sequoia(SimGeno_example, LH_HSg5, Module="par",
+#'                 args.AP=list(Discrete = TRUE),  # non-overlapping generations
+#'                 CalcLLR = FALSE,   # skip time-consuming calculation of LLR's
+#'                 Plot = FALSE)      # no summary plots when finished
 #'
 #' @export
 
@@ -261,6 +264,7 @@ MakeAgePrior <- function(Pedigree = NULL,
     if (!is.null(Discrete) && Discrete && (MaxAgeParent[1] != MaxAgeParent[2])) {
       stop("When Discrete=TRUE, MaxAgeParent must be identical for dams & sires (?)")
     }
+    if (Smooth) Smooth <- FALSE  # else maxAgePO = MaxAgeParent +2
     for (p in 1:2) {
       if (!is.na(MaxAgeParent[p]) && (MaxAgeParent[p]<0 || !is.wholenumber(MaxAgeParent[p]))) {
         stop("'MaxAgeParent' must be a positive whole number")
@@ -350,16 +354,17 @@ MakeAgePrior <- function(Pedigree = NULL,
   Ped.R <- Ped.LH[! (is.na(Ped.LH$dam) & is.na(Ped.LH$sire)), ]  # individuals with at least 1 parent (quicker)
   Ped.R <- merge(PedPolish(Ped.R[,1:3]), Ped.LH[, c("id", "BirthYear")])  # BY for dropped & re-added indivs
 
-  RelA <- GetRelA(Ped.R, patmat=TRUE, GenBack=1)
-
-  AgeDifM <- outer(Ped.R$BirthYear, Ped.R$BirthYear, "-")
+  BYV <- setNames(Ped.R$BirthYear, Ped.R$id)
+  AgeDifM <- outer(BYV, BYV, "-")
+  RelL <- GetRelA(Ped.R, patmat=TRUE, GenBack=1, List = TRUE)
   tblA.R <- sapply(c("M", "P", "FS", "MHS", "PHS"),
-                function(r) table(factor(AgeDifM[RelA[,,r]==1], levels = 0:MaxT)))
+                   function(r) table(factor(AgeDifM[ RelL[[r]] ], levels = 0:MaxT)))
+
   # maternal siblings = maternal half-siblings + full siblings
   tblA.R <- cbind(tblA.R,
                   "MS" = tblA.R[,"MHS"] + tblA.R[,"FS"],
                   "PS" = tblA.R[,"PHS"] + tblA.R[,"FS"])
-  tblA.R <- tblA.R[, c("M", "P", "FS", "MS", "PS")]
+  tblA.R <- tblA.R[, RR]
 
   # Reference: age difference distribution across all pairs of individuals
   tbl.AgeDifs <- CountAgeDif(Ped.LH$BirthYear, BYrange)  # quicker than table(outer()), function below
@@ -370,25 +375,17 @@ MakeAgePrior <- function(Pedigree = NULL,
   tblA.R["0", ] <- tblA.R["0", ] / 2   # sibs & 'X' pairs with agedif 0 counted twice
 
 
-  # return if no parents of known age ----
-  NAK.R <- apply(tblA.R, 2, sum, na.rm=TRUE)
-  if (all(NAK.R[c("M","P")]==0)) {
-    return( ReturnDefault(MaxAgePO, Return) )
-  }
-  # TODO: estimate min. MaxAgePO from sibling age difference?
-
-
-  MinPairs.AgeKnown <- 20   # minimum no. mother-offspring or father-offspring pairs w known age diff
-
-
   # update MaxAgePO ----
+  NAK.R <- apply(tblA.R, 2, sum, na.rm=TRUE)
   ParAge <- suppressWarnings(
     apply(tblA.R[, c("M", "P")], 2, function(x) max(which(x > 0)) -1) )  # 1st row = agedif 0
 
+  MinPairs.AgeKnown <- 20   # minimum no. mother-offspring or father-offspring pairs w known age diff
+  # MaxAgeParent = user input
   for (p in 1:2) {
-    if (MaxAgePO[p] == (diff(BYrange)+1) & is.na(MaxAgeParent[p]) &
-        NAK.R[p] >= MinPairs.AgeKnown) {   # ignore MaxAgePO: derived from LifeHistData
-      MaxAgePO[p] <- max(ParAge[p], na.rm=TRUE)
+    if (MaxAgePO[p] == (diff(BYrange)+1) & is.na(MaxAgeParent[p]) & NAK.R[p] >= MinPairs.AgeKnown) {
+      # ignore MaxAgePO: derived from LifeHistData
+      MaxAgePO[p] <- ParAge[p]
     } else {
       MaxAgePO[p] <- max(ParAge[p], MaxAgePO[p], na.rm=TRUE)
     }
@@ -503,11 +500,13 @@ MakeAgePrior <- function(Pedigree = NULL,
     }
     LR.RU.A.default <- MkAPdefault(MaxP = pmax(MaxAgePO, MaxAgeParent, na.rm=TRUE),  # for input > pedigree-observed
                                    Disc=Discrete)
-    nx <- max(nrow(LR.RU.A.default), nrow(LR.RU.A.par))
-    if (nrow(LR.RU.A.default) < nx) {
+    if (nrow(LR.RU.A.default) < nrow(LR.RU.A.par)) {
       LR.RU.A.default <- rbind(LR.RU.A.default,
-                               matrix(0, nx-nrow(LR.RU.A.default), 5))
+                               matrix(0, nrow(LR.RU.A.par)-nrow(LR.RU.A.default), 5))
+    } else if  (nrow(LR.RU.A.default) > nrow(LR.RU.A.par)) {
+      LR.RU.A.default <- LR.RU.A.default[1:nrow(LR.RU.A.par), ]
     }
+
     for (r in RR) {
       LR.RU.A[, r] <- W.R[r] * LR.RU.A.par[, r] + (1 - W.R[r]) * LR.RU.A.default[, r]
     }
@@ -640,7 +639,6 @@ CountAgeDif <- function(BirthYear, BYrange = range(BirthYear)) {
 #'   columns: PO-FS-HS-GP-FA-HA-U.
 #'
 #' @examples
-#' data(SeqOUT_griffin, package="sequoia")
 #' PairsG <- data.frame(ID1="i122_2007_M",
 #'                      ID2 = c("i124_2007_M", "i042_2003_F", "i083_2005_M"),
 #'                      AgeDif = c(0,4,2))
